
export interface TestMessage {
    role: string;
    content: string;
}

export class TestCompletion {
    token: string = "r:60abef69e7cd8181b146ceaba1fdbf02";
    messageList: TestMessage[] = [];
    stream: boolean = true;
    interviewEnded = false;
    recognition: any;

    
    constructor(messageList?: TestMessage[]) {
        this.messageList = messageList || this.messageList;
    }

    async sendMessage(
        messageList?: TestMessage[] | null,
        onMessage?: (content: string) => void
    ): Promise<any> {
        this.messageList = messageList || this.messageList;
        
        const body = {
            "messages": this.messageList,
            "stream": this.stream,
            "model": "fmode-4.5-128k",
            "temperature": 0.5,
            "presence_penalty": 0,
            "frequency_penalty": 0,
            "token": "Bearer " + this.token
        };

        try {
            const response = await fetch("https://server.fmode.cn/api/apig/aigc/gpt/v1/chat/completions", {
                headers: {
                    "Content-Type": "application/json"
                },
                body: JSON.stringify(body),
                method: "POST",
                mode: "cors",
                credentials: "omit"
            });

            if (!response.ok) {
                throw new Error(`HTTP error! Status: ${response.status}`);
            }

            if (!this.stream) {
                const data = await response.json();
                return data?.choices?.[0]?.message?.content;
            }

            if (!response.body) {
                throw new Error("No response body in stream mode");
            }

            const reader = response.body.getReader();
            const decoder = new TextDecoder("utf-8");
            let accumulatedContent = "";

            try {
                while (true) {
                    const { done, value } = await reader.read();
                    if (done) break;

                    const chunk = decoder.decode(value, { stream: true });
                    const lines = chunk.split('\n').filter(line => line.trim() !== '');

                    for (const line of lines) {
                        if (line.startsWith('data:') && !line.includes('[DONE]')) {
                            try {
                                const jsonStr = line.substring(5).trim();
                                const data = JSON.parse(jsonStr);
                                const content = data?.choices?.[0]?.delta?.content || '';
                                
                                if (content) {
                                    accumulatedContent += content;
                                    if (onMessage) {
                                        onMessage(accumulatedContent);
                                    }
                                }
                            } catch (e) {
                                console.error("Error parsing stream data:", e);
                            }
                        }
                    }
                }
            } finally {
                reader.releaseLock();
            }

            return accumulatedContent;
        } catch (error) {
            console.error("API request failed:", error);
            throw error;
        }
    }
}

/**
 * 生成面试评估报告
 */
export async function generateInterviewEvaluation(
    question: string,
    answer: string,
    positionRequirements: string,
    onMessage?: (content: string) => void
): Promise<{
    score: number;
    feedback: string;
    followUpQuestion: string;
    metrics: {
        expressiveness: number;
        professionalism: number;
        relevance: number;
    };
}> {
    const evaluationPrompt = `请根据以下面试对话生成评估报告：
问题：${question}
回答：${answer}
岗位要求：${positionRequirements}

请返回包含以下内容的JSON对象：
1. score: 整体评分(0-100)
2. feedback: 具体反馈意见
3. followUpQuestion: 针对回答的追问问题
4. metrics: {
    expressiveness: 表达能力评分(0-100),
    professionalism: 专业度评分(0-100),
    relevance: 岗位匹配度评分(0-100)
}

要求：
- 反馈意见应具体指出回答中的优缺点
- 追问问题应能深入了解候选人的能力
- 评分应基于岗位要求合理给出`;

    const messages: TestMessage[] = [
        {
            role: "user",
            content: evaluationPrompt
        }
    ];

    const completion = new TestCompletion(messages);
    let fullContent = "";
    
    await completion.sendMessage(null, (content) => {
        fullContent = content;
        if (onMessage) onMessage(content);
    });

    try {
        const result = extractJSON(fullContent);
        if (!result) {
            throw new Error("无法解析评估结果");
        }
        return result;
    } catch (e) {
        console.error("评估结果解析失败:", e);
        console.log("原始内容:", fullContent);
        throw new Error("生成的评估报告格式不正确");
    }
}

function extractJSON(str: string) {
    let stack = 0;
    let startIndex = -1;
    let result = null;

    for (let i = 0; i < str.length; i++) {
        if (str[i] === '{') {
            if (stack === 0) startIndex = i;
            stack++;
        } else if (str[i] === '}') {
            stack--;
            if (stack === 0 && startIndex !== -1) {
                try {
                    result = JSON.parse(str.slice(startIndex, i + 1));
                    break;
                } catch (e) {
                    startIndex = -1;
                }
            }
        }
    }
    return result;
}